@@ -3,9 +3,9 @@
  *
  * Implementation of the Tamper-Proof Device Filesystem.
  *
- * Copyright (C) 2005-2006  NTT DATA CORPORATION
+ * Copyright (C) 2005-2007  NTT DATA CORPORATION
  *
- * Version: 1.3   2006/11/11
+ * Version: 1.4   2007/04/01
  *
  * This file is applicable to both 2.4.30 and 2.6.11 and later.
  * See README.ccs for ChangeLog.
@@ -285,7 +285,7 @@ static int RegisterNodeInfo(char *buffer, struct super_block *sb)
 	char *args[MAX_ARG];
 	int i;
 	int error = -EINVAL;
-	mode_t perm; uid_t uid; gid_t gid; unsigned int major = 0, minor = 0; unsigned int flags;
+	unsigned int perm, uid, gid, flags, major = 0, minor = 0;
 	struct syaoran_sb_info *info = (struct syaoran_sb_info *) sb->s_fs_info;
 	struct dev_entry *entry;
 	memset(args, 0, sizeof(args));
@@ -297,7 +297,7 @@ static int RegisterNodeInfo(char *buffer, struct super_block *sb)
 	}
 	/* printk("<%s> <%s> <%s> <%s> <%s> <%s> <%s> <%s>\n", args[0], args[1], args[2], args[3], args[4], args[5], args[6], args[7]); */
 	if (!args[ARG_FILENAME] || !args[ARG_PERMISSION] || !args[ARG_UID] || !args[ARG_GID] || !args[ARG_DEV_TYPE] || !args[ARG_FLAGS]) goto out;
-	if (sscanf(args[ARG_PERMISSION], "%ho", &perm) != 1 || !(perm <= 0777) || sscanf(args[ARG_UID], "%u", &uid) != 1
+	if (sscanf(args[ARG_PERMISSION], "%o", &perm) != 1 || !(perm <= 0777) || sscanf(args[ARG_UID], "%u", &uid) != 1
 		|| sscanf(args[ARG_GID], "%u", &gid) != 1 || sscanf(args[ARG_FLAGS], "%u", &flags) != 1 || *(args[ARG_DEV_TYPE] + 1)) goto out;
 	switch (*args[ARG_DEV_TYPE]) {
 	case 'c':
@@ -501,7 +501,7 @@ static int Syaoran_Initialize(struct super_block *sb, void *data)
 	static int first = 1;
 	if (first) {
 		first = 0;
-		printk("SYAORAN: 1.3   2006/11/11\n");
+		printk("SYAORAN: 1.4   2007/04/01\n");
 	}
 	{
 		struct inode *inode = new_inode(sb);
@@ -598,13 +598,13 @@ static int GetLocalAbsolutePath(struct dentry *dentry, char *buffer, int buflen)
 /* Get absolute pathname of the given dentry from mount point. */
 static int local_realpath_from_dentry(struct dentry *dentry, char *newname, int newname_len)
 {
-	int error = -ENOENT;
+	int error;
 	struct dentry *d_dentry;
 	if (!dentry || !newname || newname_len <= 0) return -EINVAL;
 	d_dentry = dget(dentry);
 	/***** CRITICAL SECTION START *****/
 	spin_lock(&dcache_lock);
-	if (IS_ROOT(d_dentry) || !d_unhashed(d_dentry)) error = GetLocalAbsolutePath(d_dentry, newname, newname_len);
+	error = GetLocalAbsolutePath(d_dentry, newname, newname_len);
 	spin_unlock(&dcache_lock);
 	/***** CRITICAL SECTION END *****/
 	dput(d_dentry);
