@@ -5,7 +5,7 @@
  *
  * Copyright (C) 2005-2007  NTT DATA CORPORATION
  *
- * Version: 1.4.3   2007/11/11
+ * Version: 1.4   2007/04/01
  *
  * This file is applicable to both 2.4.30 and 2.6.11 and later.
  * See README.ccs for ChangeLog.
@@ -19,23 +19,18 @@
 
 #include <net/ip.h>
 #include <net/ipv6.h>
-#include <net/udp.h>
-#include <asm/uaccess.h>
 
 #include <linux/version.h>
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2,5,0)
 #define sk_family family
 #define sk_protocol protocol
-#define sk_type type
-#define sk_receive_queue receive_queue
 #endif
 
 #define MAX_SOCK_ADDR 128 /* net/socket.c */
 
-static inline int CheckSocketCreatePermission(int family, int type, int protocol)
+static int CheckSocketCreatePermission(int family, int type, int protocol)
 {
 	int error = 0;
-	if (segment_eq(get_fs(), KERNEL_DS)) return 0;
 	if (family == PF_INET || family == PF_INET6) {
 		switch (type) {
 		case SOCK_STREAM:
@@ -56,10 +51,9 @@ static inline int CheckSocketCreatePermission(int family, int type, int protocol
 	return error;
 }
 
-static inline int CheckSocketListenPermission(struct socket *sock)
+static int CheckSocketListenPermission(struct socket *sock)
 {
 	int error = 0;
-	if (segment_eq(get_fs(), KERNEL_DS)) return 0;
 	if (sock->type == SOCK_STREAM) {
 		switch (sock->sk->sk_family) {
 		case PF_INET:
@@ -87,11 +81,10 @@ static inline int CheckSocketListenPermission(struct socket *sock)
 	return error;
 }
 
-static inline int CheckSocketConnectPermission(struct socket *sock, struct sockaddr *addr, int addr_len)
+static int CheckSocketConnectPermission(struct socket *sock, struct sockaddr *addr, int addr_len)
 {
 	int error = 0;
 	const unsigned int type = sock->type;
-	if (segment_eq(get_fs(), KERNEL_DS)) return 0;
 	if (type == SOCK_STREAM || type == SOCK_DGRAM || type == SOCK_RAW) {
 		switch (addr->sa_family) {
 		case AF_INET6:
@@ -125,11 +118,10 @@ static inline int CheckSocketConnectPermission(struct socket *sock, struct socka
 	return error;
 }
 
-static inline int CheckSocketBindPermission(struct socket *sock, struct sockaddr *addr, int addr_len)
+static int CheckSocketBindPermission(struct socket *sock, struct sockaddr *addr, int addr_len)
 {
 	int error = 0;
 	const unsigned int type = sock->type;
-	if (segment_eq(get_fs(), KERNEL_DS)) return 0;
 	if (type == SOCK_STREAM || type == SOCK_DGRAM || type == SOCK_RAW) {
 		switch (addr->sa_family) {
 		case AF_INET6:
@@ -155,11 +147,10 @@ static inline int CheckSocketBindPermission(struct socket *sock, struct sockaddr
 	return error;
 }
 
-static inline int CheckSocketAcceptPermission(struct socket *sock, struct sockaddr *addr)
+static int CheckSocketAcceptPermission(struct socket *sock, struct sockaddr *addr)
 {
 	int error = 0;
 	int addr_len;
-	if (segment_eq(get_fs(), KERNEL_DS)) return 0;
 	switch (sock->sk->sk_family) {
 	case PF_INET:
 	case PF_INET6:
@@ -179,11 +170,10 @@ static inline int CheckSocketAcceptPermission(struct socket *sock, struct sockad
 	return error;
 }
 
-static inline int CheckSocketSendMsgPermission(struct socket *sock, struct sockaddr *addr, int addr_len)
+static int CheckSocketSendMsgPermission(struct socket *sock, struct sockaddr *addr, int addr_len)
 {
 	int error = 0;
 	const int type = sock->type;
-	if (segment_eq(get_fs(), KERNEL_DS)) return 0;
 	if (addr && (type == SOCK_DGRAM || type == SOCK_RAW)) {
 		switch (addr->sa_family) {
 		case AF_INET6:
@@ -201,87 +191,19 @@ static inline int CheckSocketSendMsgPermission(struct socket *sock, struct socka
 	return error;
 }
 
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,22)
-
-static inline struct iphdr *ip_hdr(const struct sk_buff *skb)
-{
-	return skb->nh.iph;
-}
-
-static inline struct udphdr *udp_hdr(const struct sk_buff *skb)
-{
-	return skb->h.uh;
-}
-
-static inline struct ipv6hdr *ipv6_hdr(const struct sk_buff *skb)
-{
-	return skb->nh.ipv6h;
-}
-
-#endif
-
-static inline int CheckSocketRecvDatagramPermission(struct sock *sk, struct sk_buff *skb, const unsigned int flags)
+static int CheckSocketRecvMsgPermission(struct socket *sock, struct sockaddr *addr, int addr_len)
 {
 	int error = 0;
-	const unsigned int type = sk->sk_type;
-	struct in6_addr sin6;
-	struct in_addr sin;
-	u16 port;
-		
-	if (!skb) return 0;
-
-	if (in_interrupt()) return 0;
-	
-	if (segment_eq(get_fs(), KERNEL_DS)) return 0;
-	
-	if (type != SOCK_DGRAM && type != SOCK_RAW) return 0;
-	
-	switch (sk->sk_family) {
-	case PF_INET6:
-		if (type == SOCK_DGRAM) { /* UDP IPv6 */
-			if (skb->protocol == htons(ETH_P_IP)) {
-				ipv6_addr_set(&sin6, 0, 0, htonl(0xffff), ip_hdr(skb)->saddr);
-			} else {
-				ipv6_addr_copy(&sin6, &ipv6_hdr(skb)->saddr);
-			}
-			port = udp_hdr(skb)->source;
-		} else { /* RAW IPv6 */
-			ipv6_addr_copy(&sin6, &ipv6_hdr(skb)->saddr);
-			port = htons(sk->sk_protocol);
+	const unsigned int type = sock->type;
+	if (addr && (type == SOCK_DGRAM || type == SOCK_RAW)) {
+		switch (addr->sa_family) {
+		case AF_INET6:
+			if (addr_len >= SIN6_LEN_RFC2133) error = CheckNetworkRecvMsgACL(1, type, ((struct sockaddr_in6 *) addr)->sin6_addr.s6_addr, type == SOCK_DGRAM ? ((struct sockaddr_in6 *) addr)->sin6_port : htons(sock->sk->sk_protocol));
+			break;
+		case AF_INET:
+			if (addr_len >= sizeof(struct sockaddr_in)) error = CheckNetworkRecvMsgACL(0, type, (u8 *) &((struct sockaddr_in *) addr)->sin_addr, type == SOCK_DGRAM ? ((struct sockaddr_in *) addr)->sin_port : htons(sock->sk->sk_protocol));
+			break;
 		}
-		error = CheckNetworkRecvMsgACL(1, type, (u8 *) &sin6, port);
-		break;
-	case PF_INET:
-		if (type == SOCK_DGRAM) { /* UDP IPv4 */
-			sin.s_addr = ip_hdr(skb)->saddr;
-			port = udp_hdr(skb)->source;
-		} else { /* RAW IPv4 */
-			sin.s_addr = ip_hdr(skb)->saddr;
-			port = htons(sk->sk_protocol);
-		}
-		error = CheckNetworkRecvMsgACL(0, type, (u8 *) &sin, port);
-		break;
-	}
-	if (error) {
-		/*
-		 * Remove from queue if MSG_PEEK is used so that
-		 * the head message from unwanted source in receive queue will not
-		 * prevent the caller from picking up next message from wanted source
-		 * when the caller is using MSG_PEEK flag for picking up.
-		 */
-		if (flags & MSG_PEEK) {
-			unsigned long cpu_flags;
-			spin_lock_irqsave(&sk->sk_receive_queue.lock, cpu_flags);
-			if (skb == skb_peek(&sk->sk_receive_queue)) {
-				__skb_unlink(skb, &sk->sk_receive_queue);
-				atomic_dec(&skb->users);
-			}
-			spin_unlock_irqrestore(&sk->sk_receive_queue.lock, cpu_flags);
-		}
-		/* Drop reference count. */
-		skb_free_datagram(sk, skb);
-		/* Hope less harmful than -EPERM. */
-		error = -EAGAIN;
 	}
 	return error;
 }
