@@ -4,9 +4,9 @@
  *
  * /proc interface for SAKURA and TOMOYO.
  *
- * Copyright (C) 2005-2006  NTT DATA CORPORATION
+ * Copyright (C) 2005-2007  NTT DATA CORPORATION
  *
- * Version: 1.3   2006/11/11
+ * Version: 1.4   2007/04/01
  *
  * This file is applicable to both 2.4.30 and 2.6.11 and later.
  * See README.ccs for ChangeLog.
@@ -33,7 +33,7 @@ static inline struct proc_dir_entry *PDE(const struct inode *inode)
 
 static int ccs_open(struct inode *inode, struct file *file)
 {
-	return CCS_OpenControl((const int) (PDE(inode)->data), file);
+	return CCS_OpenControl(((u8 *) PDE(inode)->data) - ((u8 *) NULL), file);
 }
 
 static int ccs_release(struct inode *inode, struct file *file)
@@ -69,7 +69,7 @@ static __init void CreateEntry(const char *name, const mode_t mode, struct proc_
 	struct proc_dir_entry *entry = create_proc_entry(name, mode, parent);
 	if (entry) {
 		entry->proc_fops = &ccs_operations;
-		entry->data = (void *) key;
+		entry->data = ((u8 *) NULL) + key;
 	}
 }
 
@@ -80,6 +80,7 @@ void __init CCSProc_Init(void)
 		*info_dir = proc_mkdir("info", ccs_dir);
 	extern void __init realpath_Init(void);
 	realpath_Init();
+	FindDomain(""); /* Set domainname of KERNEL domain. */
 	CreateEntry("query",            0600, policy_dir, CCS_POLICY_QUERY);
 #ifdef CONFIG_SAKURA
 	CreateEntry("system_policy",    0600, policy_dir, CCS_POLICY_SYSTEMPOLICY);
@@ -87,7 +88,6 @@ void __init CCSProc_Init(void)
 #ifdef CONFIG_TOMOYO
 	CreateEntry("domain_policy",    0600, policy_dir, CCS_POLICY_DOMAINPOLICY);
 	CreateEntry("exception_policy", 0600, policy_dir, CCS_POLICY_EXCEPTIONPOLICY);
-	CreateEntry("trusted_pids",     0400, info_dir, CCS_INFO_TRUSTEDPIDS);
 	CreateEntry(".domain_status",   0600, policy_dir, CCS_POLICY_DOMAIN_STATUS);
 	CreateEntry(".process_status",  0400, info_dir, CCS_INFO_PROCESS_STATUS);
 #ifdef CONFIG_TOMOYO_AUDIT
