@@ -3,9 +3,9 @@
  *
  * Implementation of the Domain-Free Mandatory Access Control.
  *
- * Copyright (C) 2005-2006  NTT DATA CORPORATION
+ * Copyright (C) 2005-2007  NTT DATA CORPORATION
  *
- * Version: 1.3.1   2006/12/08
+ * Version: 1.4   2007/04/01
  *
  * This file is applicable to both 2.4.30 and 2.6.11 and later.
  * See README.ccs for ChangeLog.
@@ -28,31 +28,36 @@ typedef struct reserved_entry {
 
 /*************************  NETWORK RESERVED ACL HANDLER  *************************/
 
-static RESERVED_ENTRY reservedport_list = { NULL, 0, 0, 0 };
+static RESERVED_ENTRY *reservedport_list = NULL;
 
 static int AddReservedEntry(const u16 min_port, const u16 max_port, const int is_delete)
 {
 	RESERVED_ENTRY *new_entry, *ptr;
-	if (!is_delete) {
-		static spinlock_t lock = SPIN_LOCK_UNLOCKED;
-		/* I don't want to add if it was already added. */
-		for (ptr = reservedport_list.next; ptr; ptr = ptr->next) {
-			if (ptr->min_port == min_port && max_port == ptr->max_port) { ptr->is_deleted = 0; return 0; }
-		}
-		if ((new_entry = (RESERVED_ENTRY *) alloc_element(sizeof(RESERVED_ENTRY))) == NULL) return -ENOMEM;
-		new_entry->min_port = min_port;
-		new_entry->max_port = max_port;
-		/***** CRITICAL SECTION START *****/
-		spin_lock(&lock);
-		for (ptr = &reservedport_list; ptr->next; ptr = ptr->next); ptr->next = new_entry;
-		spin_unlock(&lock);
-		/***** CRITICAL SECTION END *****/
-	} else {
-		for (ptr = reservedport_list.next; ptr; ptr = ptr->next) {
-			if (ptr->min_port == min_port && max_port == ptr->max_port) ptr->is_deleted = 1;
+	static DECLARE_MUTEX(lock);
+	int error = -ENOMEM;
+	for (ptr = reservedport_list; ptr; ptr = ptr->next) {
+		if (ptr->min_port == min_port && max_port == ptr->max_port) {
+			ptr->is_deleted = is_delete;
+			goto out;
 		}
 	}
-	return 0;
+	if (is_delete) {
+		error = -ENOENT;
+		goto out;
+	}
+	if ((new_entry = (RESERVED_ENTRY *) alloc_element(sizeof(RESERVED_ENTRY))) == NULL) goto out;
+	new_entry->min_port = min_port;
+	new_entry->max_port = max_port;
+	mb(); /* Instead of using spinlock. */
+	if ((ptr = reservedport_list) != NULL) {
+		while (ptr->next) ptr = ptr->next; ptr->next = new_entry;
+	} else {
+		reservedport_list = new_entry;
+	}
+	error = 0;
+ out:
+	up(&lock);
+	return error;
 }
 
 int SAKURA_MayAutobind(const u16 port)
@@ -60,7 +65,7 @@ int SAKURA_MayAutobind(const u16 port)
 	/* Must not sleep, for called inside spin_lock. */
 	RESERVED_ENTRY *ptr;
 	if (!CheckCCSFlags(CCS_SAKURA_RESTRICT_AUTOBIND)) return 0;
-	for (ptr = reservedport_list.next; ptr; ptr = ptr->next) {
+	for (ptr = reservedport_list; ptr; ptr = ptr->next) {
 		if (ptr->min_port <= port && port <= ptr->max_port && !ptr->is_deleted) return -EPERM;
 	}
 	return 0;
@@ -83,7 +88,7 @@ int AddReservedPortPolicy(char *data, const int is_delete)
 int ReadReservedPortPolicy(IO_BUFFER *head)
 {
 	RESERVED_ENTRY *ptr = (RESERVED_ENTRY *) head->read_var2;
-	if (!ptr) ptr = reservedport_list.next;
+	if (!ptr) ptr = reservedport_list;
 	while (ptr) {
 		head->read_var2 = (void *) ptr;
 		if (!ptr->is_deleted) {
