@@ -3,9 +3,9 @@
  *
  * Implementation of the Domain-Based Mandatory Access Control.
  *
- * Copyright (C) 2005-2006  NTT DATA CORPORATION
+ * Copyright (C) 2005-2007  NTT DATA CORPORATION
  *
- * Version: 1.3.1   2006/12/08
+ * Version: 1.4   2007/04/01
  *
  * This file is applicable to both 2.4.30 and 2.6.11 and later.
  * See README.ccs for ChangeLog.
@@ -37,13 +37,23 @@ char *InitAuditLog(int *len)
 	char *buf;
 	struct timeval tv;
 	struct task_struct *task = current;
-	const char *domainname = current->domain_info->domainname;
+	const char *domainname = current->domain_info->domainname->name;
 	do_gettimeofday(&tv);
 	*len += strlen(domainname) + 256;
 	if ((buf = ccs_alloc(*len)) != NULL) snprintf(buf, (*len) - 1, "#timestamp=%lu pid=%d uid=%d gid=%d euid=%d egid=%d suid=%d sgid=%d fsuid=%d fsgid=%d\n%s\n", tv.tv_sec, task->pid, task->uid, task->gid, task->euid, task->egid, task->suid, task->sgid, task->fsuid, task->fsgid, domainname);
 	return buf;
 }
 
+static unsigned int GetMaxGrantLog(void)
+{
+	return CheckCCSFlags(CCS_TOMOYO_MAX_GRANT_LOG);
+}
+
+static unsigned int GetMaxRejectLog(void)
+{
+	return CheckCCSFlags(CCS_TOMOYO_MAX_REJECT_LOG);
+}
+
 /*
  * Write audit log.
  * Caller must allocate buf with InitAuditLog().
